from dbson.serializer import Serializable

SET_COMMAND = 'set'
GET_COMMAND = 'get'
PING_COMMAND = 'ping'

COMMAND_LABEL = 'command'
COLLECTION_LABEL = 'collection'
OBJECT_NAME_LABEL = 'object'
SELECTOR_LABEL = 'selector'
DATA_LABEL = 'data'
RESULT_OF_LABEL = 'result_of'
SUCCESS_LABEL = 'success'


def construct_set(*, collection_name: str, object_name: str,
                  selector: str = "", **kwargs) -> Serializable:
    command = {
        COMMAND_LABEL: SET_COMMAND,
        COLLECTION_LABEL: collection_name,
        OBJECT_NAME_LABEL: object_name,
        **kwargs
    }
    if DATA_LABEL not in command and RESULT_OF_LABEL not in command:
        raise KeyError('`data` or `result_of` was expected')
    if selector != "":
        command[SELECTOR_LABEL] = selector
    return command


def construct_get(*, collection_name: str, object_name: str,
                  selector: str = "", **kwargs) -> Serializable:
    command = {
        COMMAND_LABEL: GET_COMMAND,
        COLLECTION_LABEL: collection_name,
        OBJECT_NAME_LABEL: object_name,
        **kwargs
    }
    if selector != "":
        command[SELECTOR_LABEL] = selector
    return command


def construct_ping(**kwargs) -> Serializable:
    command = {
        COMMAND_LABEL: PING_COMMAND,
        **kwargs
    }
    return command



import socket

import dbson.commands as commands
from dbson.deserializer import deserialize
from dbson.serializer import Serializable, serialize


class Client:

    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port

    def execute(self, command: Serializable) -> Serializable:
        return self.send(command)

    def set(self, *, collection_name: str, object_name: str,
            selector: str = "", **kwargs):
        command = commands.construct_set(collection_name=collection_name,
                                         object_name=object_name,
                                         selector=selector, **kwargs)
        return self.execute(command)

    def get(self, *, collection_name: str, object_name: str,
            selector: str = "", **kwargs):
        command = commands.construct_get(collection_name=collection_name,
                                         object_name=object_name,
                                         selector=selector, **kwargs)
        return self.execute(command)

    def ping(self, **kwargs):
        return self.execute(commands.construct_ping(**kwargs))

    def send(self, data: Serializable) -> Serializable:
        return self.send_one_tcp(data)

    def send_one_tcp(self, data: Serializable) -> Serializable:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.connect((self.host, self.port))
            buffered_io = s.makefile('wrb')
            serialize(data, buffered_io)
            buffered_io.flush()
            result = deserialize(buffered_io)
            buffered_io.flush()
            s.close()
        return result


def connect(host: str, port: int) -> Client:
    client = Client(host, port)
    response = client.ping()
    if response.get(commands.SUCCESS_LABEL, False) != True:
        raise ConnectionError('ping command failed')
    return client



import struct
from typing import BinaryIO
from dbson.serializer import Serializable


def deserialize(s: BinaryIO) -> Serializable:
    deserializers = {
        b'i': deserialize_int,
        b'r': deserialize_float,
        b's': deserialize_str,
        b'a': deserialize_array,
        b'o': deserialize_assoc,
        b't': lambda _: True,
        b'f': lambda _: False,
        b'n': lambda _: None
    }

    prefix = read_exact(s, 1)
    if prefix not in deserializers:
        raise ValueError(f"prefix 0x{prefix.hex()} not found")

    return deserializers[prefix](s)


def read_exact(s: BinaryIO, n: int) -> bytes:
    buffer = b''
    while len(buffer) < n:
        new_bytes = s.read(n - len(buffer))
        if len(new_bytes) == 0:
            raise EOFError
        buffer += new_bytes
    return buffer


def deserialize_int(s: BinaryIO) -> int:
    return int.from_bytes(read_exact(s, 4), byteorder='big')


def deserialize_float(s: BinaryIO) -> float:
    return struct.unpack('>d', read_exact(s, 8))[0]


def deserialize_len(s: BinaryIO) -> int:
    length_size: int = read_exact(s, 1)[0]
    length_bytes = read_exact(s, length_size)
    return int.from_bytes(length_bytes, byteorder='big')


def deserialize_str(s: BinaryIO) -> str:
    length = deserialize_len(s)
    return read_exact(s, length).decode('utf-8')


def deserialize_array(s: BinaryIO) -> list:
    length = deserialize_len(s)
    arr = []
    for i in range(length):
        arr.append(deserialize(s))
    return arr


deserialize_list = deserialize_array


def deserialize_assoc(s: BinaryIO) -> dict:
    length = deserialize_len(s)
    assoc = {}
    for i in range(length):
        key = deserialize(s)
        value = deserialize(s)
        assoc[key] = value
    return assoc


deserialize_dict = deserialize_assoc



from dbson.serializer import *
from dbson.deserializer import *
from dbson.client import *
from dbson.commands import *


def main():
    # a = {"name": "vasya", "age": {"sex": 12}, "parents": ["irina", "petr", None]}
    # with open('out.dd', 'wb') as f:
    #    serialize(a, f)
    # with open('out.dd', 'rb') as f:
    #    print(deserialize(f))

    conn = connect('localhost', 18889)
    conn.set(collection_name='people', object_name='sasha', data={
        'name': 'alexey',
        'parents': ['egor', 'vasya']
    })

    conn.set(collection_name='people', object_name='egor', selector='parents',
             result_of=construct_get(collection_name='people',
                                     object_name='sasha', selector='parents'))

    conn.set(collection_name='people', object_name='egor', selector='brother',
             result_of=construct_get(collection_name='people',
                                     object_name='sasha', selector='name'))

    print(conn.get(collection_name='people', object_name='egor')['data'])
    # {'brother': 'alexey', 'parents': ['egor', 'vasya']}

    print(conn.ping())


if __name__ == '__main__':
    main()



from typing import Union, List, Dict, Sized
from typing import BinaryIO
from io import BytesIO
import struct


Serializable = Union[int, float, str,
                     List['Serializable'],
                     Dict['Serializable', 'Serializable'],
                     bool, None]


def serialize(x: Serializable,
              buffer: Union[BinaryIO, None] = None) -> Union[bytes, None]:
    serializers = {
        int: serialize_int,
        float: serialize_float,
        str: serialize_str,
        list: serialize_list,
        dict: serialize_dict,
        bool: serialize_bool,
        type(None): serialize_none
    }

    if type(x) not in serializers:
        raise TypeError(f'type {type(x)} is not serializable')

    if buffer is None:
        buffer = BytesIO()
        serializers[type(x)](x, buffer)
        return buffer.read(-1)
    else:
        serializers[type(x)](x, buffer)


def serialize_int(x: int, buffer: BinaryIO):
    buffer.write(b'i' + x.to_bytes(4, byteorder='big'))


def serialize_float(x: float, buffer: BinaryIO):
    buffer.write(b'r' + struct.pack('>d', x))


def serialize_len(obj: Sized, buffer: BinaryIO):
    l_bytes = len(obj).to_bytes(8, byteorder='big')
    if not any(l_bytes[:7]):
        l_bytes = l_bytes[-1:]
    elif not any(l_bytes[:6]):
        l_bytes = l_bytes[-2:]
    elif not any(l_bytes[:4]):
        l_bytes = l_bytes[-4:]
    buffer.write(bytes([len(l_bytes)]) + l_bytes)


def serialize_str(s: str, buffer: BinaryIO):
    str_bytes = bytes(s, 'utf-8')
    buffer.write(b's')
    serialize_len(str_bytes, buffer)
    buffer.write(str_bytes)


def serialize_array(lst: List[Serializable], buffer: BinaryIO):
    buffer.write(b'a')
    serialize_len(lst, buffer)
    for el in lst:
        serialize(el, buffer)


serialize_list = serialize_array


def serialize_assoc(assoc: Dict[Serializable, Serializable],
                    buffer: BinaryIO):
    buffer.write(b'o')
    serialize_len(assoc, buffer)
    assoc_list = assoc.items()
    try:
        assoc_list = sorted(assoc_list)
    except TypeError:
        # cannot sort, several types of keys
        pass
    for key, value in assoc_list:
        serialize(key, buffer)
        serialize(value, buffer)


serialize_dict = serialize_assoc


def serialize_bool(x: bool, buffer: BinaryIO):
    buffer.write(b't' if x else b'f')


def serialize_none(_: type(None), buffer: BinaryIO):
    buffer.write(b'n')
